import org.apache.commons.lang3.StringUtils;
import java.sql.*;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.*;
import java.util.Date;
import java.util.stream.Collectors;

/**
 * zpracovava vzorecky
 * vypoctene vysledky aplikuje na data
 *
 * @author Petr Holicky
 */
public class EquationParser {
    /* pripojeni k databazi */
    final Connection connection;
    /* ukazatel na pocitanou radku/zaznam */
    Object currRow;
    /* urcuje aktualni uroven vypoctu */
    Level currLevel;

    /**
     * konstruktor
     * nastavi pripojeni k databazi - connection
     * @param connection pripojeni k databazi
     */
    public EquationParser(Connection connection) {
        this.connection = connection;
    }

    /**
     * rozdeli predany vzorecek podle mezery a preda parseFunction()
     * ziskany vysledek zapise do conjoined
     * pokud conjoined neobsahuje nespocitana data -> oznaci radek za spocitany
     * @param conjoined sloupec conjoined
     * @param eq vzorecek
     * @param row aktualni radek/zaznam
     * @param level aktualni uroven
     * @return conjoined na ktery byl aplikovan vzorecek
     */
    public String applyEquation(String conjoined, HashMap<String, Object> eq, Object row, Level level) {
        this.currRow = row;
        this.currLevel = level;

        String equation = eq.get("eq").toString();
        String[] equationParts = equation.split(" ");
        String result = parseFunction(equationParts);
        if(result.equals("") || result.equals("NaN")) {
            result = "";
            Client.recalcsFailedCount++;
        }
        String name = eq.get("name").toString().toLowerCase()+"=";
        if(conjoined.contains(name)) {
            if(!result.equals("")) {
                String value = StringUtils.substringAfter(conjoined, name);
                value = StringUtils.substringBefore(value, ";");
                conjoined = conjoined.replace(name+value+";", name+result+";");
            }
        } else {
            conjoined += name+(result)+";";
        }
        if(!conjoined.contains("=;")) {
            switch (level) {
                case DATA: if(Client.new_data_finished.size() > (Integer) currRow) Client.new_data_finished.set((Integer) currRow, true);
                           else Client.new_data_finished.add((Integer) currRow, true);break;
                case JOB: Client.jobs_finished.put((Key) currRow, true);break;
                case DAY: Client.daysFromJobs_finished.put((Key) currRow, true);break;
                case SUM: Client.summ_finished.put((Integer) currRow, true);break;
            }
        } else {
            switch (level) {
                case DATA: if(Client.new_data_finished.size() > (Integer) currRow) Client.new_data_finished.set((Integer) currRow, false);
                           else Client.new_data_finished.add((Integer) currRow, false);break;
                case JOB: Client.jobs_finished.put((Key) currRow, false);break;
                case DAY: Client.daysFromJobs_finished.put((Key) currRow, false);break;
                case SUM: Client.summ_finished.put((Integer) currRow, false);break;
            }
        }

        return conjoined;
    }

    /**
     * zpracuje vzorecek rekurzivne, po castech
     * obsahuje switch pojmenovanych funkci
     * pokud neni funkce -> hodnota/konstanta - preda prislusne getVariables podle aktualni urovne
     * popis funkci v dokumentaci
     * @param eq vzorecek
     * @return vysledek vzorecku, pokud nelze spocitat -> vraci prazdny retezec ""
     */
    public String parseFunction(String[] eq) {
        double a,b,c,d,e;
        try {
            switch (eq[0]) {
                //SKALARNI FUNKCE
                case "(": return parseFunction(Arrays.copyOfRange(eq, 1, eq.length));
                case ")": return "";
                case "ADD":
                case "+": a = checkForBrackets(eq,1);
                          b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                          c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                          return String.valueOf(b + c);
                case "SUB":
                case "-": a = checkForBrackets(eq,1);
                          b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                          c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                          return String.valueOf(b - c);
                case "MUL":
                case "*": a = checkForBrackets(eq,1);
                          b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                          c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                          return String.valueOf(b * c);
                case "DIV":
                case "/": a = checkForBrackets(eq,1);
                          b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                          c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                          return String.valueOf(b / c);
                case "ABS": a = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                            return String.valueOf(Math.abs(a));
                case "MOD": a = checkForBrackets(eq,1);
                            b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                            c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                            return String.valueOf(b % c);
                case "MAX": a = checkForBrackets(eq,1);
                            b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                            c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                            return String.valueOf(Math.max(b, c));
                case "MIN": a = checkForBrackets(eq,1);
                            b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                            c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                            return String.valueOf(Math.min(b, c));
                case "AND": a = checkForBrackets(eq,1);
                            b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                            c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                            if(b>0.5) b = 1;else b = 0;
                            if(c>0.5) c = 1;else c = 0;
                            return String.valueOf(((int) b) & ((int) c));
                case "OR": a = checkForBrackets(eq,1);
                           b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                           c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                           if(b>0.5) b = 1;else b = 0;
                           if(c>0.5) c = 1;else c = 0;
                           return String.valueOf(((int) b) | ((int) c));
                case "SHL": a = checkForBrackets(eq,1);
                            b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                            c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                            return String.valueOf(Integer.rotateLeft((int) b, (int) c));
                case "SHR": a = checkForBrackets(eq,1);
                            b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                            c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                            return String.valueOf(Integer.rotateRight((int) b, (int) c));
                case "EQ":
                case "=": a = checkForBrackets(eq,1);
                          b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                          c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                          if(b == c) return "1"; else return "0";
                case "EQS": a = checkForBrackets(eq,1);
                            if(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)).equals(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length))))return "1"; else return "0";
                case "GT":
                case ">": a = checkForBrackets(eq,1);
                          b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                          c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                          if(b > c) return "1"; else return "0";
                case "GET":
                case ">=": a = checkForBrackets(eq,1);
                           b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                           c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                           if(b >= c) return "1"; else return "0";
                case "LT":
                case "<": a = checkForBrackets(eq,1);
                          b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                          c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                          if(b < c) return "1"; else return "0";
                case "LET":
                case "<=": a = checkForBrackets(eq,1);
                           b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                           c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                           if(b <= c) return "1"; else return "0";
                case "POW": a = checkForBrackets(eq,1);
                            b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                            c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                            if(b>0) return String.valueOf(Math.pow(b, c)); else return "";
                case "IF": a = checkForBrackets(eq,1);
                           if(a>1) {
                               c = checkForBrackets(eq, (int) (a+1));
                           } else {
                               b = checkForBrackets(eq, (int) (a+1));
                               c = checkForBrackets(eq, (int) (b+1));
                           }
                           d = checkForBrackets(eq, (int) (c+1));
                           String condition = parseFunction(Arrays.copyOfRange(eq, 1, eq.length));
                           if(condition.equals("1")) {
                               return parseFunction(Arrays.copyOfRange(eq, (int) (c+1), eq.length));
                           } else if(condition.equals("0")) {
                               return parseFunction(Arrays.copyOfRange(eq, (int) (d+1), eq.length));
                           }
                           else return "";
                case "INR": a = checkForBrackets(eq,1);
                            b = checkForBrackets(eq, (int) (a+1));
                            c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                            d = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                            e = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (b+1), eq.length)));
                            if(c > d && c < e) return "1"; else return "0";
                case "TOR": a = checkForBrackets(eq,1);
                            b = checkForBrackets(eq, (int) (a+1));
                            c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                            d = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                            e = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (b+1), eq.length)));
                            if(c > d && c < e) return String.valueOf(c); else return String.valueOf(Math.abs(c-d)<Math.abs(c-e)?d:e);
                case "IFNN": a = checkForBrackets(eq,1);
                             String isString = parseFunction(Arrays.copyOfRange(eq, 1, eq.length));
                             if(isNumeric(isString)) return isString; else return parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length));
                case "IFNA": a = checkForBrackets(eq,1);
                             String isEmpty = parseFunction(Arrays.copyOfRange(eq, 1, eq.length));
                             if(isEmpty.equals("")) return parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)); else return isEmpty;
                case "ISN": if(isNumeric(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)))) return "1"; else return "0";
                case "ISA": if(!parseFunction(Arrays.copyOfRange(eq, 1, eq.length)).equals("")) return "1"; else return "0";
                case "DDIF": a = checkForBrackets(eq,1);
                             String ddif_sa = parseFunction(Arrays.copyOfRange(eq, 1, eq.length));
                             Timestamp ddif_a = Timestamp.valueOf(ddif_sa);
                             Timestamp ddif_b = Timestamp.valueOf(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                             return String.valueOf((ddif_a.getTime() - ddif_b.getTime())/1000);
                case "DADD": a = checkForBrackets(eq,1);
                             Timestamp dadd = Timestamp.valueOf(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                             b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                             Calendar cal = Calendar.getInstance();
                             cal.setTimeInMillis(dadd.getTime());
                             cal.add(Calendar.SECOND, (int) b);
                             return new Timestamp(cal.getTime().getTime()).toString();
                case "DWEEK": Timestamp dweek = Timestamp.valueOf(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                              return String.valueOf(dweek.toLocalDateTime().toLocalDate().getDayOfWeek().getValue());
                case "GPSDIF": a = checkForBrackets(eq,1);
                               String[] gps_a = parseFunction(Arrays.copyOfRange(eq, 1, eq.length)).split(",");
                               String[] gps_b = parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)).split(",");
                               b = Double.parseDouble(gps_a[0]);
                               c = Double.parseDouble(gps_a[1]);
                               d = Double.parseDouble(gps_b[0]);
                               e = Double.parseDouble(gps_b[1]);
                               double oneDegreeInMeters = 40040000d/360;
                               if(a < -180 || b < -180 || c < -180 || e < -180) return "";
                               else return String.valueOf(Math.sqrt(Math.sqrt(oneDegreeInMeters*(d-b)) + Math.sqrt(d*(e-c)*Math.cos(b*(Math.PI/180)))));
                case "GPSST": a = checkForBrackets(eq,1);
                              b = checkForBrackets(eq, (int) (a+1));
                              c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                              String[] gpsst_a = parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)).split(",");
                              String[] gpsst_b = parseFunction(Arrays.copyOfRange(eq, (int) (b+1), eq.length)).split(",");
                              a = Double.parseDouble(gpsst_a[0]);
                              b = Double.parseDouble(gpsst_a[1]);
                              d = Double.parseDouble(gpsst_b[0]);
                              e = Double.parseDouble(gpsst_b[1]);
                              double oneDegreeInMeters2 = 40040000d/360;
                              double dist = Math.sqrt(Math.sqrt(oneDegreeInMeters2*(d-a)) + Math.sqrt(d*(e-b)*Math.cos(a*(Math.PI/180))));
                              if(dist < c) return Arrays.toString(gpsst_b); else return Arrays.toString(gpsst_a);
                case "ADDS": a = checkForBrackets(eq,1);
                             return parseFunction(Arrays.copyOfRange(eq, 1, eq.length)) + parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length));
                case "I2S": return parseFunction(Arrays.copyOfRange(eq, 1, eq.length));
                case "GOTH": a = checkForBrackets(eq,1);
                             b = checkForBrackets(eq, (int) (a+1));
                             c = checkForBrackets(eq, (int) (b+1));
                             double goth_a = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                             double goth_b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                             double goth_c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (b+1), eq.length)));
                             double goth_d = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (c+1), eq.length)));
                             if(goth_a < 0) {
                                 if(goth_b < goth_c) return "1";
                                 if(goth_b > goth_d && goth_c < goth_d) return "1";
                             }
                             else if(goth_a > 0) {
                                 if(goth_b > goth_c) return "1";
                                 if(goth_b < goth_d && goth_c < goth_d) return "1";
                             }
                             else {
                                 if(goth_b < goth_d && goth_c > goth_d) return "1";
                                 if (goth_b > goth_d && goth_c < goth_d) return "1";
                                 return "0";
                             }

                case "ROUND": return String.valueOf(Math.round(Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)))));
                case "ROUNDX": a = checkForBrackets(eq,1);
                               b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                               c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                               if(c >= 0) {
                                   if(c % 10 == 0) {
                                       return String.valueOf(Math.round(b * c) / c);
                                   } else if(c == 0) {
                                       return String.valueOf(Math.round(b));
                                   } else {
                                       c = c * 10;
                                       return String.valueOf(Math.round(b * c) / c);
                                   }
                               } else return String.valueOf(b);
                case "ENUM": a = checkForBrackets(eq,1);
                             return parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)).split("&")[Integer.parseInt(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)))];
                case "TOE": a = checkForBrackets(eq,1);
                            return parseFunction(Arrays.copyOfRange(eq, 1, eq.length))+"&"+parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length));
                case "TABLE": a = checkForBrackets(eq,1);
                              b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                              String table = parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length));
                              List<Double> table_x = new ArrayList<>();
                              List<Double> table_y = new ArrayList<>();
                              String[] table_axis = table.split("\\|");
                              for(String s : table_axis) {
                                  String[] xy = s.split(",");
                                  table_x.add(Double.parseDouble(xy[0]));
                                  table_y.add(Double.parseDouble(xy[1]));
                              }
                              //qi = b , i,o = table_x/y , qo = d , k = c, q = table_q
                              int table_q;
                              if(table_x.size() != table_y.size() || table_x.size()<= 0) {
                                  d = b;
                              } else if(table_x.size() == 1) {
                                  if(table_x.get(0) != 0) {
                                      c = table_y.get(0)/table_x.get(0);
                                      d = b * c;
                                  } else {
                                      d = b;
                                  }
                              } else {
                                  table_q = 0;
                                  while(table_q < table_x.size() && b > table_x.get(table_q))
                                      table_q++;
                                  if(table_q == 0) table_q = 1;
                                  if(table_q >= table_x.size()) table_q = table_x.size()-1;
                                  if((table_x.get(table_q) - table_x.get(table_q-1)) > 1e-8) {
                                      c = (table_y.get(table_q) - table_y.get(table_q-1)) / (table_x.get(table_q) - table_x.get(table_q-1));
                                  } else {
                                      c = 0;
                                  }
                                  d = table_y.get(table_q-1) + (b - table_x.get(table_q-1)) * c;
                              }
                              return String.valueOf(d);

                case "RND": return String.valueOf(Math.random());
                case "RNDN": return String.valueOf(new Random().nextGaussian());
                case "TRANS": a = checkForBrackets(eq,1);
                              b = checkForBrackets(eq, (int) (a+1));
                              c = checkForBrackets(eq, (int) (b+1));
                              d = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (c+1), eq.length)));
                              c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (b+1), eq.length)));
                              b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                              a = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                              if(c >= d) {
                                  if(a < c) return String.valueOf(a);
                                  else if(b > c) return String.valueOf(b);
                                  else return String.valueOf(c);
                              }
                              if(a <= c) {return String.valueOf(a);}
                              else if(a >= d) {return String.valueOf(b);}
                              else {
                                  double x = (a-c)/(d-c);
                                  if(x < 0) return "0";
                                  if(x > 1) return "1";
                                  return String.valueOf((1 - x) * a + x * b);
                              }
                case "HILO": a = checkForBrackets(eq,1);
                             b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                             String a_hilo = parseFunction(Arrays.copyOfRange(eq, 1, eq.length));
                             List<Double> hilo = parseHistogramAxis(a_hilo);
                             if(b < 0) {
                                 return String.valueOf(hilo.get((int) (hilo.size()-b)));
                             } else {
                                 for(int i = 0; i<hilo.size()-1; i++) {
                                     if(b>hilo.get(i) && b<hilo.get(i+1)) {
                                         return String.valueOf(hilo.get(i));
                                     }
                                 }
                                 return "";
                             }
                case "HIHO": a = checkForBrackets(eq,1);
                             b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                             String a_hiho = parseFunction(Arrays.copyOfRange(eq, 1, eq.length));
                             List<Double> hiho = parseHistogramAxis(a_hiho);
                             if(b < 0) {
                                 return String.valueOf(hiho.get((int) (hiho.size()-b)));
                             } else {
                                 for(int i = 0; i<hiho.size()-1; i++) {
                                     if(b>hiho.get(i) && b<hiho.get(i+1)) {
                                         return String.valueOf(hiho.get(i+1));
                                     }
                                 }
                                 return "";
                             }
                case "HIIX": a = checkForBrackets(eq,1);
                             b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                             String a_hiix = parseFunction(Arrays.copyOfRange(eq, 1, eq.length));
                             List<Double> hiix = parseHistogramAxis(a_hiix);
                             for(int i = 0; i<hiix.size()-1; i++) {
                                 if(b>hiix.get(i) && b<hiix.get(i+1)) {
                                     return String.valueOf(i);
                                 }
                             }
                             return "";


                // SUMACNI FUNKCE
                case "SNRM": return String.valueOf(numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length))).stream().mapToDouble((x) -> x).average().orElseThrow(NumberFormatException::new));
                case "SMED": List<Double> smed = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                             Collections.sort(smed);
                             if (smed.size() % 2 == 0)
                                 return String.valueOf((smed.get(smed.size()/2) + smed.get(smed.size()/2 - 1) /2));
                             else
                                 return String.valueOf(smed.get(smed.size()/2));
                case "SMIN": List<Double> smin = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                             return String.valueOf(Collections.min(smin));
                case "SMAX": List<Double> smax = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                             return String.valueOf(Collections.max(smax));
                case "SSUM": List<Double> ssum = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                             return String.valueOf(ssum.stream().mapToDouble((x) -> x).sum());
                case "SSUMX": List<String> ssumx = stringListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                              double ssumx_out = 0;
                              for(String x : ssumx) {
                                  if(isNumeric(x))
                                    ssumx_out += Double.parseDouble(x);
                              }
                              return String.valueOf(ssumx_out);
                case "SSUMN": List<String> ssumn = stringListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                              double ssumn_out = 0;
                              for(String s : ssumn) {
                                  if(isNumeric(s))
                                      ssumn_out += Double.parseDouble(s);
                                  else
                                      return "";
                              }
                              return String.valueOf(ssumn_out);
                case "SBEG": List<String> sbeg = stringListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                             if(!sbeg.isEmpty())
                                return sbeg.get(0);
                             else return "";
                case "SEND": List<String> send = stringListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                             if(!send.isEmpty())
                                return String.valueOf(send.get(send.size()-1));
                             else return "";
                case "SBEGN": a = checkForBrackets(eq,1);
                              b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                              List<String> sbegn = stringListFromArrayString(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                              if(sbegn.size()>b)
                                return sbegn.get((int) b);
                              else return "";
                case "SENDN": a = checkForBrackets(eq,1);
                              b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                              List<String> sendn = stringListFromArrayString(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                              return sendn.get(sendn.size()-1-(int) b);
                case "SBEGNR": a = checkForBrackets(eq,1);
                               b = checkForBrackets(eq, (int) (a+1));
                               c = checkForBrackets(eq, (int) (b+1));
                               d = checkForBrackets(eq, (int) (c+1));
                               int sbegnr_index = Integer.parseInt(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                               List<Double> sbegnr = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                               int sbegnr_low = Integer.parseInt(parseFunction(Arrays.copyOfRange(eq, (int) (b+1), eq.length)));
                               int sbegnr_high = Integer.parseInt(parseFunction(Arrays.copyOfRange(eq, (int) (c+1), eq.length)));
                               for(int i = sbegnr_index; i<sbegnr.size(); i++) {
                                   Double sbegnr_value = sbegnr.get(i);
                                   if(sbegnr_value > sbegnr_low && sbegnr_value < sbegnr_high)
                                       return String.valueOf(sbegnr_value);
                               }
                               return parseFunction(Arrays.copyOfRange(eq, (int) (d+1), eq.length));
                case "SENDNR": a = checkForBrackets(eq,1);
                               b = checkForBrackets(eq, (int) (a+1));
                               c = checkForBrackets(eq, (int) (b+1));
                               d = checkForBrackets(eq, (int) (c+1));
                               int sendnr_index = Integer.parseInt(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                               List<Double> sendnr = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                               int sendnr_low = Integer.parseInt(parseFunction(Arrays.copyOfRange(eq, (int) (b+1), eq.length)));
                               int sendnr_high = Integer.parseInt(parseFunction(Arrays.copyOfRange(eq, (int) (c+1), eq.length)));
                               for(int i = sendnr.size()-1-sendnr_index; i >= 0; i--) {
                                   Double sbegnr_value = sendnr.get(i);
                                   if(sbegnr_value > sendnr_low && sbegnr_value <sendnr_high)
                                       return String.valueOf(sbegnr_value);
                                }
                               return parseFunction(Arrays.copyOfRange(eq, (int) (d+1), eq.length));
                case "SSUMD": List<Double> ssumd = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                              double ssumd_sum = 0;
                              for(int i = 0; i<ssumd.size()-1; i++) {
                                  ssumd_sum += Math.abs(ssumd.get(i) - ssumd.get(i+1));
                              }
                              return String.valueOf(ssumd_sum);
                case "SSUMP": List<Double> ssump = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                              double ssump_sum = 0;
                              for(int i = 0; i<ssump.size()-1; i++) {
                                  double ssump_value = ssump.get(i) - ssump.get(i+1);
                                  if(ssump_value >= 0)
                                      ssump_sum += ssump_value;
                              }
                              return String.valueOf(ssump_sum);
                case "SSUMM": List<Double> ssumm = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                              double ssumm_sum = 0;
                              for(int i = 0; i<ssumm.size()-1; i++) {
                                  double ssumm_value = ssumm.get(i) - ssumm.get(i+1);
                                  if(ssumm_value < 0)
                                      ssumm_sum += Math.abs(ssumm_value);
                              }
                              return String.valueOf(ssumm_sum);
                case "SCNT": List<String> scnt = stringListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                             return String.valueOf(scnt.size());
                case "SCNT0": List<String> scnt0 = stringListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                              int scnt0_count = 0;
                              for(String s : scnt0) {
                                  if(s.equals(""))
                                      scnt0_count++;
                              }
                              return String.valueOf(scnt0_count);
                case "SCNTS": List<String> scnts = stringListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                              int scnts_count = 0;
                              for(String s : scnts) {
                                  if(!s.equals(""))
                                      scnts_count++;
                              }
                              return String.valueOf(scnts_count);
                case "SCNTN": List<String> scntn = stringListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                              int scntn_count = 0;
                              for(String s : scntn) {
                                  if(isNumeric(s))
                                      scntn_count++;
                              }
                              return String.valueOf(scntn_count);
                case "SCNTNR": a = checkForBrackets(eq,1);
                               b = checkForBrackets(eq, (int) (a+1));
                               List<Double> scntnr = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                               c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                               d = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (b+1), eq.length)));
                               double scntnr_counter = 0;
                               if(c <= d) {
                                   for(Double x : scntnr) {
                                       if(x > c && x < d)
                                           scntnr_counter++;
                                   }
                               } else {
                                   for(Double x : scntnr) {
                                       if(x > c || x < d)
                                           scntnr_counter++;
                                   }
                               }
                               return String.valueOf(scntnr_counter);

                case "SDATCTRL": a = checkForBrackets(eq,1);
                                 b = checkForBrackets(eq, (int) (a+1));
                                 c = checkForBrackets(eq, (int) (b+1));
                                 d = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                                 List<Double> sdatctrl_out = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                                 List<Double> sdatctrl_in = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, (int) (b+1), eq.length)));
                                 e = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (c+1), eq.length)));
                                 double mx, mn;
                                 for(int q = 0; q<sdatctrl_out.size(); q++) {
                                     mx = sdatctrl_in.get(q);
                                     mn = sdatctrl_in.get(q);
                                     for(int w=-2; w<=2; w++) {
                                         if((q+w) >= 0 && (q+w) < sdatctrl_in.size())
                                              mx = Math.max(mx, sdatctrl_in.get(q+w));
                                    }
                                     sdatctrl_out.set(q, 0.0);
                                     if(d == 0) {
                                         if ((mx-mn)>e)
                                             sdatctrl_out.set(q, 1.0);
                                      } else if(d == 1) {
                                         if (mn>e)
                                             sdatctrl_out.set(q, 1.0);
                                     } else if(d == 2) {
                                         if (mx<e)
                                             sdatctrl_out.set(q, 1.0);
                                     } else if(d == 3) {
                                         sdatctrl_out.set(q, mx-mn);
                                     }
                                 }
                                 if(!sdatctrl_out.isEmpty())
                                     return doubleListToString(sdatctrl_out);
                                 else return "";
                case "SLIST": a = checkForBrackets(eq,1);
                              b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                              List<String> slist = stringListFromArrayString(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                              if(b == 0) {
                                  slist =  slist.stream().distinct().collect(Collectors.toList());
                                  return slist.toString().replace("[", "").replace("]", "").replace(" ", "");
                              }
                              else if(b == 1) {
                                  return slist.toString().replace("[", "").replace("]", "").replace(" ", "");
                              }
                              else if(b == 2) {
                                  slist =  slist.stream().distinct().collect(Collectors.toList());
                                  Collections.sort(slist);
                                  return slist.toString().replace("[", "").replace("]", "").replace(" ", "");
                              }
                              else if(b == 3) {
                                  Collections.sort(slist);
                                  return slist.toString().replace("[", "").replace("]", "").replace(" ", "");
                              } else return "";
                case "SA": return parseFunction(Arrays.copyOfRange(eq, 1, eq.length));

                //VEKTOROVE FUNKCE
                case "VSEL": a = checkForBrackets(eq,1);
                             List<Double> vsel_a = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                             List<Double> vsel_b = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                             List<Double> vsel_out = new ArrayList<>();
                             for(int i = 0; i<vsel_a.size() && i<vsel_b.size(); i++) {
                                 if(vsel_a.get(i) > 0.5)
                                     vsel_out.add(vsel_b.get(i));
                             }
                             return doubleListToString(vsel_out);
                case "VISR": a = checkForBrackets(eq,1);
                             b = checkForBrackets(eq, (int) (a+1));
                             List<Double> visr = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                             c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                             d = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (b+1), eq.length)));
                             List<Integer> visr_out = new ArrayList<>();
                             for(Double visr_d : visr) {
                                 if(visr_d > c && visr_d < d)
                                     visr_out.add(1);
                                 else
                                     visr_out.add(0);
                             }
                             return intListToString(visr_out);
                case "VAND": a = checkForBrackets(eq,1);
                             List<Double> vand_a = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                             List<Integer> vand_a_int = new ArrayList<>();
                             List<Double> vand_b = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                             List<Integer> vand_b_int = new ArrayList<>();
                             for(int i = 0; i<vand_a.size(); i++) {
                                 if(vand_a.get(i) > 0.5)
                                     vand_a_int.add(1);
                                 else
                                     vand_a_int.add(0);
                                 if(vand_b.get(i) > 0.5)
                                     vand_b_int.add(1);
                                 else
                                     vand_b_int.add(0);
                             }
                             List<Integer> vand_out = new ArrayList<>();
                             for(int i = 0; i<vand_a.size(); i++) {
                                 vand_out.add(vand_a_int.get(i) & vand_b_int.get(i));
                             }
                             return intListToString(vand_out);
                case "VOR": a = checkForBrackets(eq,1);
                            List<Double> vor_a = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                            List<Integer> vor_a_int = new ArrayList<>();
                            List<Double> vor_b = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                            List<Integer> vor_b_int = new ArrayList<>();
                            for(int i = 0; i<vor_a.size(); i++) {
                                if(vor_a.get(i) > 0.5)
                                    vor_a_int.add(1);
                                else
                                    vor_a_int.add(0);
                                if(vor_b.get(i) > 0.5)
                                    vor_b_int.add(1);
                                else
                                    vor_b_int.add(0);
                            }
                            List<Integer> vor_out = new ArrayList<>();
                            for(int i = 0; i<vor_a.size(); i++) {
                                vor_out.add(vor_a_int.get(i) | vor_b_int.get(i));
                            }
                            return intListToString(vor_out);
                case "VNOT": List<Double> vnot = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                             List<Integer> vnot_out = new ArrayList<>();
                             for(Double aDouble : vnot) {
                                 if (aDouble > 0.5)
                                     vnot_out.add(1);
                                 else
                                     vnot_out.add(0);
                             }
                             vnot_out.replaceAll(x -> ~x);
                             return intListToString(vnot_out);
                case "VBEX": a = checkForBrackets(eq,1);
                             List<Double> vbex = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                             List<Integer> vbex_int = new ArrayList<>();
                             for(Double x : vbex) vbex_int.add(x.intValue());
                             b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq,(int) (a+1), eq.length)));
                             if(b > 0) {
                                 for(int i = 0; i<vbex.size(); i++) {
                                     if(vbex_int.get(i) == 1) {
                                         for(int j = i+1; j<=i+b && j<vbex_int.size(); j++) {
                                             vbex_int.set(j, 1);
                                         }
                                         for(int j = i-1; j>=i-b && j>=0; j++) {
                                             vbex_int.set(j, 1);
                                         }
                                     }
                                 }
                                 return intListToString(vbex_int);
                             }
                             else if(b < 0) {
                                 for(int i = 0; i<vbex.size(); i++) {
                                     if(vbex_int.get(i) == 0) {
                                         for(int j = i+1; j<=i+b && j<vbex_int.size(); j++) {
                                             vbex_int.set(j, 0);
                                         }
                                         for(int j = i-1; j>=i-b && j>=0; j++) {
                                             vbex_int.set(j, 0);
                                         }
                                     }
                                 }
                                 return intListToString(vbex_int);
                             }
                             else
                                 return "";
                case "VLAD": a = checkForBrackets(eq,1);
                             b = checkForBrackets(eq, (int) (a+1));
                             List<Double> vlad_a = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq,  1, eq.length)));
                             List<Double> vlad_b = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq,  (int) (a+1), eq.length)));
                             List<Integer> vlad_out = new ArrayList<>();
                             c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq,(int) (b+1), eq.length)));
                             for(int i = 0; i<vlad_a.size(); i++) {
                                 if(vlad_a.get(i) < (vlad_b.get(i) - c)) {
                                     vlad_out.add(1);
                                 } else vlad_out.add(0);
                             }
                             return intListToString(vlad_out);
                case "VBEG": a = checkForBrackets(eq,1);
                             b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq,1, eq.length)));
                             List<String> vbeg = stringListFromArrayString(parseFunction(Arrays.copyOfRange(eq,  (int) (a+1), eq.length)));
                             List<String> vbeg_out = new ArrayList<>();
                             for(int i = 0; i < b && i<vbeg.size(); i++) {
                                 vbeg_out.add(vbeg.get(i));
                             }
                             return vbeg_out.toString().replace("[", "").replace("]", "").replace(" ", "");
                case "VEND": a = checkForBrackets(eq,1);
                             b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq,1, eq.length)));
                             List<String> vend = stringListFromArrayString(parseFunction(Arrays.copyOfRange(eq,  (int) (a+1), eq.length)));
                             List<String> vend_out = new ArrayList<>();
                             for(int i = vend.size()-1; i>b && i>=0; i--) {
                                 vend_out.add(vend.get(i));
                             }
                             return vend_out.toString().replace("[", "").replace("]", "").replace(" ", "");
                case "VIDX": a = checkForBrackets(eq,1);
                             b = checkForBrackets(eq, (int) (a+1));
                             c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq,1, eq.length)));
                             d = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq,(int) (a+1), eq.length)));
                             List<String> vidx = stringListFromArrayString(parseFunction(Arrays.copyOfRange(eq,  (int) (b+1), eq.length)));
                             List<String> vidx_out = new ArrayList<>();
                             for(int i = (int) c; i<d && i<vidx.size(); i--) {
                                 vidx_out.add(vidx.get(i));
                             }
                             return vidx_out.toString().replace("[", "").replace("]", "").replace(" ", "");
                case "VMID":  a = checkForBrackets(eq,1);
                            b = checkForBrackets(eq, (int) (a+1));
                            c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq,1, eq.length)));
                            d = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq,(int) (a+1), eq.length)));
                            List<String> vmid = stringListFromArrayString(parseFunction(Arrays.copyOfRange(eq,  (int) (b+1), eq.length)));
                            List<String> vmid_out = new ArrayList<>();
                            c = (vmid.size()/2d) + c;
                            d = (vmid.size()/2d) + d;
                            for(int i = (int) c; i<=d && i<vmid.size(); i++) {
                                vmid_out.add(vmid.get(i));
                            }
                            return vmid_out.toString().replace("[", "").replace("]", "").replace(" ", "");
                case "VNUM": List<String> vnum = stringListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                             List<Double> vnum_out = new ArrayList<>();
                             for(String s : vnum) {
                                 if(isNumeric(s))
                                     vnum_out.add(Double.parseDouble(s));
                             }
                             return doubleListToString(vnum_out);
                case "VMUL": a = checkForBrackets(eq,1);
                             List<Double> vmul_a = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq,  1, eq.length)));
                             List<Double> vmul_b = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq,  (int) (a+1), eq.length)));
                             List<Double> vmul_out = new ArrayList<>();
                             for(int i = 0; i<vmul_a.size(); i++) {
                                 vmul_out.add(vmul_a.get(i) * vmul_b.get(i));
                             }
                             return doubleListToString(vmul_out);
                case "VDIV": a = checkForBrackets(eq,1);
                             List<Double> vdiv_a = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq,  1, eq.length)));
                             List<Double> vdiv_b = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq,  (int) (a+1), eq.length)));
                             List<Double> vdiv_out = new ArrayList<>();
                             for(int i = 0; i<vdiv_a.size(); i++) {
                                 vdiv_out.add(vdiv_a.get(i) / vdiv_b.get(i));
                             }
                             return doubleListToString(vdiv_out);
                case "VADD": a = checkForBrackets(eq,1);
                             List<Double> vadd_a = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq,  1, eq.length)));
                             List<Double> vadd_b = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq,  (int) (a+1), eq.length)));
                             List<Double> vadd_out = new ArrayList<>();
                             for(int i = 0; i<vadd_a.size(); i++) {
                                 vadd_out.add(vadd_a.get(i) + vadd_b.get(i));
                             }
                             return doubleListToString(vadd_out);
                case "VMULC": a = checkForBrackets(eq,1);
                              double vmulc_con = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                              List<Double> vmulc = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                              vmulc.replaceAll(x -> x*vmulc_con);
                              return  doubleListToString(vmulc);
                case "VADDC": a = checkForBrackets(eq,1);
                              double vaddc_con = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                              List<Double> vaddc = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                              vaddc.replaceAll(x -> x+vaddc_con);
                              return  doubleListToString(vaddc);
                case "VSSS": a = checkForBrackets(eq,1);
                             b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                             List<Double> vsss = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                             List<Double> shifted = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                             List<Double> vsss_out = new ArrayList<>();
                             if(b > 0) {
                                 for(int i = 0; i<vsss.size(); i++) {
                                     if(i < b)
                                         shifted.add(0.0);
                                     else
                                         shifted.add(vsss.get(i-(int)b));
                                 }
                             }
                             else if(b < 0) {
                                 for(int i = 0; i<vsss.size(); i++) {
                                     if(i >= vsss.size()+b)
                                         shifted.add(0.0);
                                     else
                                         shifted.add(vsss.get(i-(int)b));
                                 }
                             } else return "";
                             for(int i = 0; i<vsss.size(); i++) {
                                 vsss_out.add(vsss.get(i) - shifted.get(i));
                             }
                             return doubleListToString(vsss_out);
                case "VSHIFT": a = checkForBrackets(eq,1);
                               b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                               List<Double> vshift = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                               List<Double> vshift_out = new ArrayList<>();
                               if(b > 0) {
                                   for(int i = (int) b; i<vshift.size(); i++) {
                                       vshift_out.add(vshift.get(i-(int)b));
                                   }
                                   return doubleListToString(vshift_out);
                               }
                               else if(b < 0) {
                                   for(int i = 0; i<vshift.size()+b; i++) {
                                       vshift_out.add(vshift.get(i-(int)b));
                                   }
                                   return doubleListToString(vshift_out);
                               }
                               else
                                   return doubleListToString(vshift);
                case "VT": return parseFunction(Arrays.copyOfRange(eq, 1, eq.length));
                case "VA": return parseFunction(Arrays.copyOfRange(eq, 1, eq.length));
                case "VAN": return parseFunction(Arrays.copyOfRange(eq, 1, eq.length));
                case "HISTS": a = checkForBrackets(eq,1);
                              b = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                              List<Double> hists = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                              Double hists_out = (hists.stream().mapToDouble((x) -> x).sum())/b;
                              return String.valueOf(hists_out);
                case "HISTD": a = checkForBrackets(eq,1);
                              b = checkForBrackets(eq,(int) (a+1));
                              c = Double.parseDouble(parseFunction(Arrays.copyOfRange(eq, (int) (b+1), eq.length)));
                              List<Double> histd_a = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                              List<Double> histd_b = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                              List<Double> histd_out = new ArrayList<>();
                              if(histd_a.size() == histd_b.size()) {
                                  for(int i = 0; i<histd_a.size(); i++) {
                                      histd_out.add((histd_a.get(i) / histd_b.get(i))/c);
                                  }
                              } else return "";
                              return doubleListToString(histd_out);
                case "HIST": a = checkForBrackets(eq,1);
                             b = checkForBrackets(eq,(int) (a+1));
                             List<Double> hist = parseHistogramAxis(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                             List<Double> hist_values = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                             List<Double> hist_increments = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, (int) (b+1), eq.length)));
                             double[] hist_out = new double[hist.size()/2];
                             if(hist_increments.isEmpty()) {
                                 for(Double x : hist_values) {
                                     for(int i = 0; i<hist.size()-2; i+=2) {
                                         if(x > hist.get(i) && x < hist.get(i+1)){
                                             hist_out[i/2] += 1;
                                         }
                                     }
                                 }
                             } else {
                                 if(hist_increments.size() == hist_out.length) {
                                     for(Double x : hist_values) {
                                         for(int i = 0; i<hist.size()-2; i+=2) {
                                             if(x > hist.get(i) && x < hist.get(i+1)){
                                                 hist_out[i/2] += hist_increments.get(i/2);
                                             }
                                         }
                                     }
                                 }
                             }
                             List<Double> hist_out_list = Arrays.stream(hist_out).boxed().collect(Collectors.toList());
                             return doubleListToString(hist_out_list);
                case "HIST2": a = checkForBrackets(eq,1);
                              b = checkForBrackets(eq,(int) (a+1));
                              c = checkForBrackets(eq,(int) (b+1));
                              d = checkForBrackets(eq,(int) (c+1));
                              List<Double> hist2_x = parseHistogramAxis(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                              List<Double> hist2_y = parseHistogramAxis(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                              List<Double> hist2_values_x = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, (int) (b+1), eq.length)));
                              List<Double> hist2_values_y = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, (int) (c+1), eq.length)));
                              List<Double> hist2_increments = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, (int) (d+1), eq.length)));
                              double[] hist2_out = new double[(hist2_x.size()/2) * (hist2_y.size()/2)];
                              if(hist2_increments.isEmpty()) {
                                  if(hist2_values_x.size() == hist2_values_y.size()) {
                                      for(int x = 0; x<hist2_values_x.size(); x++) {
                                          int pos_x = -1;
                                          int pos_y = -1;
                                          for(int i = 0; i<hist2_x.size()-2; i+=2) {
                                              if(hist2_values_x.get(x) > hist2_x.get(i) && hist2_values_x.get(x) < hist2_x.get(i+1)){
                                                  pos_x = i/2;
                                                  break;
                                              }
                                          }
                                          for(int i = 0; i<hist2_y.size()-2; i+=2) {
                                              if(hist2_values_y.get(x) > hist2_y.get(i) && hist2_values_y.get(x) < hist2_y.get(i+1)){
                                                  pos_y = i/2;
                                                  break;
                                              }
                                          }
                                          if(pos_x != -1 || pos_y != -1) {
                                              hist2_out[pos_y * (hist2_x.size()/2) + pos_x] += 1;
                                          }
                                      }
                                  }
                              } else {
                                  if(hist2_increments.size() == hist2_out.length && hist2_values_x.size() == hist2_values_y.size()) {
                                      for(int x = 0; x<hist2_values_x.size(); x++) {
                                          int pos_x = -1;
                                          int pos_y = -1;
                                          for(int i = 0; i<hist2_x.size()-2; i+=2) {
                                              if(hist2_values_x.get(x) > hist2_x.get(i) && hist2_values_x.get(x) < hist2_x.get(i+1)){
                                                  pos_x = i/2;
                                                  break;
                                              }
                                          }
                                          for(int i = 0; i<hist2_y.size()-2; i+=2) {
                                              if(hist2_values_y.get(x) > hist2_y.get(i) && hist2_values_y.get(x) < hist2_y.get(i+1)){
                                                  pos_y = i/2;
                                                  break;
                                              }
                                          }
                                          if(pos_x != -1 || pos_y != -1) {
                                              hist2_out[pos_y * (hist2_x.size()/2) + pos_x] += hist2_increments.get(x);
                                          }
                                      }
                                  }
                              }
                              List<Double> hist2_out_list = Arrays.stream(hist2_out).boxed().collect(Collectors.toList());
                              return doubleListToString(hist2_out_list);
                case "HISTIF": a = checkForBrackets(eq,1);
                               b = checkForBrackets(eq,(int) (a+1));
                               c = checkForBrackets(eq,(int) (b+1));
                               List<Double> histif = parseHistogramAxis(parseFunction(Arrays.copyOfRange(eq, 1, eq.length)));
                               List<Double> histif_values = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, (int) (a+1), eq.length)));
                               List<Double> histif_increments = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, (int) (b+1), eq.length)));
                               List<Double> histif_condition = numberListFromArrayString(parseFunction(Arrays.copyOfRange(eq, (int) (c+1), eq.length)));
                               double[] histif_out = new double[histif.size()/2];
                               if(histif_increments.isEmpty()) {
                                   for(int x = 0; x<histif_values.size(); x++) {
                                       for(int i = 0; i<histif.size()-2; i+=2) {
                                           if(histif_values.get(x) > histif.get(i) && histif_values.get(x) < histif.get(i+1)){
                                               if(histif_condition.get(x) > 0.5)
                                                   histif_out[i/2] += 1;
                                           }
                                       }
                                   }
                               } else {
                                   if(histif_increments.size() == histif_out.length) {
                                       for(int x = 0; x<histif_values.size(); x++) {
                                           for(int i = 0; i<histif.size()-2; i+=2) {
                                               if(histif_values.get(x) > histif.get(i) && histif_values.get(x) < histif.get(i+1)){
                                                   if(histif_condition.get(x) > 0.5)
                                                       histif_out[i/2] += histif_increments.get(i/2);
                                               }
                                           }
                                       }
                                   }
                               }
                               List<Double> histif_out_list = Arrays.stream(histif_out).boxed().collect(Collectors.toList());
                               return doubleListToString(histif_out_list);

                default: if(eq[0].contains("$") || eq[0].contains("#")) {
                              if(currLevel == Level.DATA) return getVariablesForData(eq[0]);
                              else if(currLevel == Level.JOB) return getVariablesForJobs(eq[0]);
                              else if(currLevel == Level.DAY) return getVariablesForDays(eq[0]);
                              else if(currLevel == Level.SUM) return getVariablesForSumTot(eq[0]);
                              else return "";
                          } else return eq[0];
            }
        } catch (Exception ex) {
            return "";
        }
    }

    /**
     * ziska hodnotu podle zadaneho jmena
     * vyznam formatu jmena v dokumentaci
     * pro uroven dat
     * @param eq nazev promenne/kanalu nebo konstanty
     * @return hodnota
     */
    private String getVariablesForData(String eq) {
        while(eq.contains("$") && StringUtils.substringAfter(eq, "$").charAt(0) != '$') {
            String variable = StringUtils.substringAfter(eq, "$");
            variable = StringUtils.substringBefore(variable, " ");
            String value;
            String column = checkVariableName(variable);
            if(isAllUpperCase(variable)) {
                //aktualni job
                HashMap<String, Object> job = Client.recalcedJobs.get(new Key((Timestamp) Client.new_data.get((Integer) currRow).get("data_time"), (Integer) Client.new_data.get((Integer) currRow).get("id")));
                if(job == null || job.isEmpty()) {
                    return "";
                }
                value = (String) job.get(column);
                variable = variable.toLowerCase();
            } else {
                // aktualni data
                value = Client.new_data.get((Integer) currRow).get(column).toString();
            }
            if(value.equals("") || !value.contains(variable))
                return "";
            if(value.contains(variable+"=")) {
                value = value.split(variable+"=")[1];
                value = removeLeadingAndEndingFromVariable(value);
                eq = StringUtils.replaceOnceIgnoreCase(eq, "$"+variable, value);
            } else return "";
        }
        while(eq.contains("$$")) {
            String variable = StringUtils.substringAfter(eq, "$$");
            variable = StringUtils.substringBefore(variable, " ");
            String value = "";
            String column = checkVariableName(variable);
            if(isAllUpperCase(variable)) {
                //predchozi job
                HashMap<String, Object> previousJob = getPreviousJob((Integer) Client.new_data.get((Integer) currRow).get("id"), new Key((Timestamp) Client.new_data.get((Integer) currRow).get("data_time"), (Integer) Client.new_data.get((Integer) currRow).get("id")));
                if(previousJob == null || previousJob.isEmpty()) {
                    return "";
                }
                value = previousJob.get(column).toString();
                variable = variable.toLowerCase();
            } else {
                //predchozi data
                for(int i = ((int) currRow)-1; i>=0; i--){
                    if(Client.new_data.get(i).get("id").equals(Client.new_data.get((Integer) currRow).get("id"))){
                        value = Client.new_data.get(i).get(column).toString();
                        break;
                    }
                }
            }
            if(value.equals("") || !value.contains(variable))
                return "";
            if(value.contains(variable+"=")) {
                value = value.split(variable+"=")[1];
                value = removeLeadingAndEndingFromVariable(value);
                eq = StringUtils.replaceOnceIgnoreCase(eq, "$$"+variable, value);
            } else return "";

        }
        while(eq.contains("#")) {
            eq = getConstantsForData(eq);
        }
        return eq;
    }

    /**
     * ziska hodnotu podle zadaneho jmena
     * vyznam formatu jmena v dokumentaci
     * pro uroven jobu
     * @param eq nazev promenne/kanalu nebo konstanty
     * @return hodnota
     */
    private String getVariablesForJobs(String eq) {
        String value;
        //jeden $
        while(eq.contains("$") && StringUtils.substringAfter(eq, "$").charAt(0) != '$') {
            String variable = StringUtils.substringAfter(eq, "$");
            variable = StringUtils.substringBefore(variable, " ");
            String column = checkVariableName(variable);
            if(isAllUpperCase(variable)) {
                // aktualni job
                HashMap<String, Object> job = Client.recalcedJobs.get((Key)currRow);
                if(job == null || job.isEmpty()) {
                    return "";
                }
                value = job.get(column).toString();
                variable = variable.toLowerCase();
                if(value.equals("") || !value.contains(variable))
                    return "";
                if(value.contains(variable+"=")) {
                    value = value.split(variable+"=")[1];
                    value = removeLeadingAndEndingFromVariable(value);
                    eq = StringUtils.replaceOnceIgnoreCase(eq, "$"+variable, value);
                } else return "";
            } else {
                //pole dat aktuálního jobu
                List<HashMap<String, Object>> job = Client.jobs.get((Key)currRow);
                String[] data = new String[job.size()];
                for(int i = 0; i<job.size(); i++) {
                    value =  job.get(i).get(column).toString();
                    if(value.equals("") || !value.contains(variable))
                        data[i] = "";
                    if(value.contains(variable+"=")) {
                        value = value.split(variable+"=")[1];
                        value = removeLeadingAndEndingFromVariable(value);
                        data[i] = value;
                    }
                }
                eq = Arrays.toString(data);
            }
        }
        while(eq.contains("$$")) {
            String variable = StringUtils.substringAfter(eq, "$$");
            variable = StringUtils.substringBefore(variable, " ");
            String column = checkVariableName(variable);
            if(isAllUpperCase(variable)) {
                // predchozi job
                variable = variable.toLowerCase();
                HashMap<String, Object> previousJob = getPreviousJob((Integer) Client.jobs.get((Key)currRow).get(0).get("id"), (Key) currRow);
                if(previousJob == null || previousJob.isEmpty()) {
                    return "";
                }
                value = previousJob.get(column).toString();
                if(value.equals("") || !value.contains(variable))
                    return "";
                if(value.contains(variable+"=")) {
                    value = value.split(variable+"=")[1];
                    value = removeLeadingAndEndingFromVariable(value);
                    eq = StringUtils.replaceOnceIgnoreCase(eq, "$$"+variable, value);
                } else return "";
            } else {
                //pole dat předchozího jobu
                List<HashMap<String, Object>> previousJob = getPreviousJobArray((Integer) Client.jobs.get((Key)currRow).get(0).get("id"), (Key) currRow);
                if(previousJob == null || previousJob.isEmpty()) {
                    return "";
                }
                String[] data = new String[previousJob.size()];
                for(int i = 0; i<previousJob.size(); i++) {
                    value = previousJob.get(i).get(column).toString();
                    if(value.equals("") || !value.contains(variable))
                        data[i] = "";
                    if(value.contains(variable+"=")) {
                        value = value.split(variable+"=")[1];
                        value = removeLeadingAndEndingFromVariable(value);
                        data[i] = value;
                    }
                }
                eq = Arrays.toString(data);
            }
        }
        while(eq.contains("#")) {
            eq = getConstantsForJobs(eq);
        }
        return eq;
    }

    /**
     * ziska hodnotu podle zadaneho jmena
     * vyznam formatu jmena v dokumentaci
     * pro uroven dnu
     * @param eq nazev promenne/kanalu nebo konstanty
     * @return hodnota
     */
    private String getVariablesForDays(String eq) {
        String value;
        //jeden $
        while(eq.contains("$") && StringUtils.substringAfter(eq, "$").charAt(0) != '$' && StringUtils.substringAfter(eq, "$").charAt(1) != '$') {
            String variable = StringUtils.substringAfter(eq, "$");
            variable = StringUtils.substringBefore(variable, " ");
            String column = checkVariableName(variable);
            if(isAllUpperCase(variable)) {
                // aktualni day $KANAL
                HashMap<String, Object> day = Client.recalcedDays.get((Key) currRow);
                if(day == null || day.isEmpty()) {
                    return "";
                }
                value = day.get(column).toString();
                variable = variable.toLowerCase();
                if(value.equals("") || !value.contains(variable))
                    return "";
                if(value.contains(variable+"=")) {
                    value = value.split(variable+"=")[1];
                    value = removeLeadingAndEndingFromVariable(value);
                    eq = StringUtils.replaceOnceIgnoreCase(eq, "$"+variable, value);
                } else return "";
            } else {
                //pole jobu v aktualni den $kanal
                List<HashMap<String, Object>> day = Client.daysFromJobs.get((Key) currRow);
                String[] data = new String[day.size()];
                for(int i = 0; i<day.size(); i++) {
                    value = day.get(i).get(column).toString();
                    if(value.equals("") || !value.contains(variable))
                        data[i] = "";
                    if(value.contains(variable+"=")) {
                        value = value.split(variable+"=")[1];
                        value = removeLeadingAndEndingFromVariable(value);
                        data[i] = value;
                    }
                }
                eq = Arrays.toString(data);
            }
        }
        while(eq.contains("$$")) {
            String variable = StringUtils.substringAfter(eq, "$$");
            variable = StringUtils.substringBefore(variable, " ");
            String column = checkVariableName(variable);
            if(isAllUpperCase(variable)) {
                // predchozi day $$KANAL
                variable = variable.toLowerCase();
                HashMap<String, Object> previousDay = getPreviousDay();
                if(previousDay == null || previousDay.isEmpty()) {
                    return "";
                }
                value = previousDay.get(column).toString();
                if(value.equals("") || !value.contains(variable))
                    return "";
                if(value.contains(variable+"=")) {
                    value = value.split(variable+"=")[1];
                    value = removeLeadingAndEndingFromVariable(value);
                    eq = StringUtils.replaceOnceIgnoreCase(eq, "$$"+variable, value);
                } else return "";
            } else {
                //pole jobu predchoziho dne $$kanal
                List<HashMap<String, Object>> previousDay = getPreviousDayArray();
                if(previousDay == null || previousDay.isEmpty()) {
                    return "";
                }
                String[] data = new String[previousDay.size()];
                for(int i = 0; i<previousDay.size(); i++) {
                    value = previousDay.get(i).get(column).toString();
                    if(value.equals("") || !value.contains(variable))
                        data[i] = "";
                    if(value.contains(variable+"=")) {
                        value = value.split(variable+"=")[1];
                        value = removeLeadingAndEndingFromVariable(value);
                        data[i] = value;
                    }
                }
                eq = Arrays.toString(data);
            }
        }
        while(eq.contains("$d$")) {
            String variable = StringUtils.substringAfter(eq, "$d$");
            variable = StringUtils.substringBefore(variable, " ");
            String column = checkVariableName(variable);
            //pole dat v aktualnim dni $d$kanal
            List<HashMap<String, Object>> job = Client.daysFromData.get((Key) currRow);
            String[] data = new String[job.size()];
            for(int i = 0; i<job.size(); i++) {
                value = job.get(i).get(column).toString();
                if(value.equals("") || !value.contains(variable))
                    data[i] = "";
                if(value.contains(variable+"=")) {
                    value = value.split(variable+"=")[1];
                    value = removeLeadingAndEndingFromVariable(value);
                    data[i] = value;
                }
            }
            eq = Arrays.toString(data);
        }
        while(eq.contains("#")) {
            eq = getConstantsForDays(eq);
        }
        return eq;
    }

    /**
     * ziska hodnotu podle zadaneho jmena
     * vyznam formatu jmena v dokumentaci
     * pro uroven sumaci
     * @param eq nazev promenne/kanalu nebo konstanty
     * @return hodnota
     */
    private String getVariablesForSumTot(String eq) {
        String value;
        //jeden $
        while(eq.contains("$") && StringUtils.substringAfter(eq, "$").charAt(0) != '$' && StringUtils.substringAfter(eq, "$").charAt(1) != '$') {
            String variable = StringUtils.substringAfter(eq, "$");
            variable = StringUtils.substringBefore(variable, " ");
            String column = checkVariableName(variable);
            if(isAllUpperCase(variable)) {
                // aktualni sum $KANAL
                HashMap<String, Object> day = Client.calcedSumms.get((int) currRow);
                if(day == null || day.isEmpty()) {
                    return "";
                }
                value = (String) day.get(column);
                variable = variable.toLowerCase();
                if(value.equals("") || !value.contains(variable))
                    return "";
                if(value.contains(variable+"=")) {
                    value = value.split(variable+"=")[1];
                    value = removeLeadingAndEndingFromVariable(value);
                    eq = StringUtils.replaceOnceIgnoreCase(eq, "$"+variable, value);
                } else return "";
            } else {
                //pole dni v rozsahu $kanal
                String[] data = new String[Client.summs.get((int) currRow).size()];
                for(int i = 0; i<Client.summs.get((int) currRow).size(); i++) {
                    value = ((String) Client.summs.get((int) currRow).get(i).get(column));
                    if(value.equals("") || !value.contains(variable))
                        data[i] = "";
                    if(value.contains(variable+"=")) {
                        value = value.split(variable+"=")[1];
                        value = removeLeadingAndEndingFromVariable(value);
                        data[i] = value;
                    }
                }
                eq = Arrays.toString(data);
            }
        }
        while(eq.contains("$$")) {
            String variable = StringUtils.substringAfter(eq, "$$");
            variable = StringUtils.substringBefore(variable, " ");
            String column = checkVariableName(variable);
            //den pred rozsahem $$kanal
            HashMap<String, Object> previousDay = getDayBeforeRange();
            if(previousDay == null || previousDay.isEmpty()) {
                return "";
            }
            value = (String) previousDay.get(column);
            if(value.equals("") || !value.contains(variable))
                return "";
            if(value.contains(variable)) {
                value = value.split(variable+"=")[1];
                value = removeLeadingAndEndingFromVariable(value);
                eq = StringUtils.replaceOnceIgnoreCase(eq, "$$"+variable, value);
            } else return "";
        }
        while(eq.contains("$j$")) {
            String variable = StringUtils.substringAfter(eq, "$j$");
            variable = StringUtils.substringBefore(variable, " ");
            String column = checkVariableName(variable);
            // joby v pocitanem rozsahu $j$kanal
            List<HashMap<String, Object>> jobs = getJobsInRange();
            String[] data = new String[jobs.size()];
            for(int i = 0; i<jobs.size(); i++) {
                value = ((String) jobs.get(i).get(column));
                if(value.equals("") || !value.contains(variable))
                    data[i] = "";
                if(value.contains(variable)) {
                    value = value.split(variable+"=")[1];
                    value = removeLeadingAndEndingFromVariable(value);
                    data[i] = value;
                }
            }
            eq = Arrays.toString(data);
        }
        while(eq.contains("#")) {
            eq = getConstantsForSumTot(eq);
        }
        return eq;
    }

    /**
     * pseudokonstanty pro data
     * pokud neni pseudokonstantou -> preda getTableConstants()
     * @param eq nazev konstanty
     * @return hodnota
     */
    private String getConstantsForData(String eq) {
        switch (eq) {
            case "#GID": eq = Client.new_data.get((Integer) currRow).get("id").toString();break;
            case "#MT": Timestamp previous_mt = null;
                        for(int i = (Integer) currRow - 1; i>=0; i--) {
                            if(Client.new_data.get(i).get("id") == Client.new_data.get((Integer) currRow).get("id")) {
                                previous_mt = (Timestamp) Client.new_data.get(i).get("datetime");
                                break;
                            }
                        }
                        if(previous_mt == null)
                            eq = "0";
                        else
                            eq = String.valueOf((((Timestamp) Client.new_data.get((Integer) currRow).get("datetime")).getTime() - (previous_mt.getTime()))/1000);break;
            case "#VMT": Key vmt_key = new Key((Timestamp) Client.new_data.get((Integer) currRow).get("data_time"), (Integer) Client.new_data.get((Integer) currRow).get("id"));
                         List<Double> vmt = new ArrayList<>();
                         List<HashMap<String, Object>> vmt_job = Client.jobs.get(vmt_key);
                         for(int i = 0; i<vmt_job.size()-1; i++) {
                             vmt.add((((Timestamp) vmt_job.get(i + 1).get("datetime")).getTime() - ((Timestamp) vmt_job.get(i).get("datetime")).getTime())/1000d);
                         }
                         eq = doubleListToString(vmt);break;
            case "#JT": eq = String.valueOf((((Timestamp) Client.new_data.get((Integer) currRow).get("datetime")).getTime() - ((Timestamp) Client.new_data.get((Integer) currRow).get("data_time")).getTime())/1000);break;
            case "#REC": Key rec_key = new Key((Timestamp) Client.new_data.get((Integer) currRow).get("data_time"), (Integer) Client.new_data.get((Integer) currRow).get("id"));
                         if(Client.jobs.containsKey(rec_key)) {
                             eq = String.valueOf(Client.jobs.get(rec_key).indexOf(Client.new_data.get((Integer) currRow)));
                         } else { eq = ""; }break;
            case "#JB": eq = String.valueOf(Client.new_data.get((Integer) currRow).get("data_time"));break;
            case "#JE": Key je_key = new Key((Timestamp) Client.new_data.get((Integer) currRow).get("data_time"), (Integer) Client.new_data.get((Integer) currRow).get("id"));
                        eq = String.valueOf(Client.recalcedJobs.get(je_key).get("data_time"));break;
            case "#WB": String wb = getTableConstants(eq, Client.new_data.get(0).get("id").toString());
                        if(wb.equals("")) return "";
                        StringBuilder wb_sb = new StringBuilder(wb);
                        wb_sb.insert(2, ':');
                        wb_sb.insert(5, ':');
                        String wb_date =  Client.new_data.get((Integer) currRow).get("data_time").toString().split(" ")[0];
                        Timestamp wb_ts = Timestamp.valueOf(wb_date+" "+wb_sb);
                        Calendar wb_cal = Calendar.getInstance();
                        wb_cal.setTime(wb_ts);
                        eq = new Timestamp(wb_cal.getTime().getTime()).toString();break;
            case "#WE": String we = getTableConstants("#WB", Client.new_data.get(0).get("id").toString());
                        if(we.equals("")) return "";
                        StringBuilder we_sb = new StringBuilder(we);
                        we_sb.insert(2, ':');
                        we_sb.insert(5, ':');
                        String we_date =  Client.new_data.get((Integer) currRow).get("data_time").toString().split(" ")[0];
                        Timestamp we_ts = Timestamp.valueOf(we_date+" "+we_sb);
                        Calendar we_cal = Calendar.getInstance();
                        we_cal.setTime(we_ts);
                        we_cal.add(Calendar.DAY_OF_WEEK, 1);
                        we_cal.add(Calendar.SECOND, -1);
                        eq = new Timestamp(we_cal.getTime().getTime()).toString();break;
            case "#NULL": eq = "";break;
            default: eq = getTableConstants(eq, Client.new_data.get(0).get("id").toString());break;
        }
        return eq;
    }

    /**
     * pseudokonstanty pro joby
     * pokud neni pseudokonstantou -> preda getTableConstants()
     * @param eq nazev konstanty
     * @return hodnota
     */
    private String getConstantsForJobs(String eq) {
        switch (eq) {
            case "#GID": eq = Client.jobs.get((Key) currRow).get(0).get("id").toString();break;
            case "#MT": if(!Client.recalcedJobs.containsKey((Key) currRow))  return  "";
                        eq = String.valueOf((((Timestamp)Client.recalcedJobs.get((Key) currRow).get("data_time")).getTime() - ((Timestamp)Client.recalcedJobs.get((Key) currRow).get("datetime")).getTime())/1000);break;
            case "#VMT": List<Double> vmt = new ArrayList<>();
                         List<HashMap<String, Object>> vmt_job = Client.jobs.get((Key) currRow);
                         for(int i = 0; i<vmt_job.size()-1; i++) {
                             vmt.add((((Timestamp) vmt_job.get(i + 1).get("datetime")).getTime() - ((Timestamp) vmt_job.get(i).get("datetime")).getTime())/1000d);
                         }
                         eq = doubleListToString(vmt);break;
            case "#JT": if(!Client.recalcedJobs.containsKey((Key) currRow))  return  "";
                        eq = String.valueOf((((Timestamp) Client.recalcedJobs.get((Key) currRow).get("datetime")).getTime() - ((Timestamp) Client.recalcedJobs.get((Key) currRow).get("datetime")).getTime())/1000);break;
            case "#RECS": eq = String.valueOf(Client.jobs.get((Key) currRow).size());break;
            case "#JB": eq = ((Key) currRow).getData_time().toString();break;
            case "#JE": if(!Client.recalcedJobs.containsKey((Key) currRow)) return  "";
                        eq = Client.recalcedJobs.get((Key) currRow).get("data_time").toString();break;
            case "#WB": String wb = getTableConstants(eq, String.valueOf(((Key) currRow).getId()));
                        if(wb.equals("")) return "";
                        StringBuilder wb_sb = new StringBuilder(wb);
                        wb_sb.insert(2, ':');
                        wb_sb.insert(5, ':');
                        String wb_date =  ((Key) currRow).getData_time().toString().split(" ")[0];
                        Timestamp wb_ts = Timestamp.valueOf(wb_date+" "+wb_sb);
                        Calendar wb_cal = Calendar.getInstance();
                        wb_cal.setTime(wb_ts);
                        eq = new Timestamp(wb_cal.getTime().getTime()).toString();break;
            case "#WE": String we = getTableConstants("#WB", Client.jobs.get((Key) currRow).get(0).get("id").toString());
                        if(we.equals("")) return "";
                        StringBuilder we_sb = new StringBuilder(we);
                        we_sb.insert(2, ':');
                        we_sb.insert(5, ':');
                        String we_date =  ((Key) currRow).getData_time().toString().split(" ")[0];
                        Timestamp we_ts = Timestamp.valueOf(we_date+" "+we_sb);
                        Calendar we_cal = Calendar.getInstance();
                        we_cal.setTime(we_ts);
                        we_cal.add(Calendar.DAY_OF_WEEK, 1);
                        we_cal.add(Calendar.SECOND, -1);
                        eq = new Timestamp(we_cal.getTime().getTime()).toString();break;
            case "#NULL": eq = "";break;
            default: eq = getTableConstants(eq, Client.jobs.get((Key) currRow).get(0).get("id").toString());break;
        }
        return eq;
    }

    /**
     * pseudokonstanty pro dny
     * pokud neni pseudokonstantou -> preda getTableConstants()
     * @param eq nazev konstanty
     * @return hodnota
     */
    private String getConstantsForDays(String eq) {
        switch (eq) {
            case "#GID": eq = Client.daysFromJobs.get((Key) currRow).get(0).get("id").toString();break;
            case "#VMT": List<Double> vmt = new ArrayList<>();
                         List<HashMap<String, Object>> vmt_day = Client.daysFromJobs.get((Key) currRow);
                         for(HashMap<String, Object> day : vmt_day) {
                             vmt.add((((Timestamp) day.get("data_time")).getTime() - ((Timestamp) day.get("datetime")).getTime()) / 1000d);
                         }
                         eq = doubleListToString(vmt);break;
            case "#JT": eq = String.valueOf((((Timestamp) Client.recalcedDays.get((Key) currRow).get("datetime")).getTime() - ((Timestamp) Client.recalcedDays.get((Key) currRow).get("datetime")).getTime())/1000);break;
            case "#RECS": eq = String.valueOf(Client.daysFromJobs.get((Key) currRow).size());break;
            case "#WB": String wb = getTableConstants(eq, Client.daysFromJobs.get((Key)currRow).get(0).get("id").toString());
                        if(wb.equals("")) return "";
                        StringBuilder wb_sb = new StringBuilder(wb);
                        wb_sb.insert(2, ':');
                        wb_sb.insert(5, ':');
                        String wb_date =  ((Key) currRow).getData_time().toString().split(" ")[0];
                        Timestamp wb_ts = Timestamp.valueOf(wb_date+" "+wb_sb);
                        Calendar wb_cal = Calendar.getInstance();
                        wb_cal.setTime(wb_ts);
                        eq = new Timestamp(wb_cal.getTime().getTime()).toString();break;
            case "#WE": String we = getTableConstants("#WB", Client.daysFromJobs.get((Key)currRow).get(0).get("id").toString());
                        if(we.equals("")) return "";
                        StringBuilder we_sb = new StringBuilder(we);
                        we_sb.insert(2, ':');
                        we_sb.insert(5, ':');
                        String we_date =  ((Key) currRow).getData_time().toString().split(" ")[0];
                        Timestamp we_ts = Timestamp.valueOf(we_date+" "+we_sb);
                        Calendar we_cal = Calendar.getInstance();
                        we_cal.setTime(we_ts);
                        we_cal.add(Calendar.DAY_OF_WEEK, 1);
                        we_cal.add(Calendar.SECOND, -1);
                        eq = new Timestamp(we_cal.getTime().getTime()).toString();break;
            case "#NULL": eq = "";break;
            default: eq = getTableConstants(eq, Client.daysFromJobs.get((Key)currRow).get(0).get("id").toString());break;
        }
        return eq;
    }

    /**
     * pseudokonstanty pro sumace
     * pokud neni pseudokonstantou -> preda getTableConstants()
     * @param eq nazev konstanty
     * @return hodnota
     */
    private String getConstantsForSumTot(String eq) {
        switch (eq) {
            case "#GID": eq = currRow.toString();break;
            case "#MT": eq = String.valueOf((Client.stopStamp.getTime()-Client.startStamp.getTime())/1000);break;
            case "#VMT": List<Double> vmt = new ArrayList<>();
                         List<HashMap<String, Object>> vmt_day = Client.summs.get((int) currRow);
                         for(HashMap<String, Object> day : vmt_day) {
                             vmt.add((((Timestamp) day.get("data_time")).getTime() - ((Timestamp) day.get("datetime")).getTime()) / 1000d);
                         }
                         eq = doubleListToString(vmt);break;
            case "#RECS": eq = String.valueOf(Client.summs.get((int) currRow).size());break;
            case "#JB": eq = Client.summs.get((int) currRow).get(0).get("datetime").toString();break;
            case "#JE": eq = Client.summs.get((int) currRow).get(Client.summs.get((int) currRow).size()-1).get("data_time").toString();break;
            case "#WB": String wb = getTableConstants(eq, currRow.toString());
                        if(wb.equals("")) return "";
                        StringBuilder wb_sb = new StringBuilder(wb);
                        wb_sb.insert(2, ':');
                        wb_sb.insert(5, ':');
                        String wb_date =  Client.summs.get((Integer) currRow).get(0).get("datetime").toString().split(" ")[0];
                        Timestamp wb_ts = Timestamp.valueOf(wb_date+" "+wb_sb);
                        Calendar wb_cal = Calendar.getInstance();
                        wb_cal.setTime(wb_ts);
                        eq = new Timestamp(wb_cal.getTime().getTime()).toString();break;
            case "#WE": String we =  getTableConstants("#WB", currRow.toString());
                        if(we.equals("")) return "";
                        StringBuilder we_sb = new StringBuilder(we);
                        we_sb.insert(2, ':');
                        we_sb.insert(5, ':');
                        String we_date =  Client.summs.get((Integer) currRow).get(0).get("datetime").toString().split(" ")[0];
                        Timestamp we_ts = Timestamp.valueOf(we_date+" "+we_sb);
                        Calendar we_cal = Calendar.getInstance();
                        we_cal.setTime(we_ts);
                        we_cal.add(Calendar.DAY_OF_WEEK, 1);
                        we_cal.add(Calendar.SECOND, -1);
                        eq = new Timestamp(we_cal.getTime().getTime()).toString();break;
            case "#NULL": eq = "";break;
            default: eq = getTableConstants(eq, currRow.toString());break;
        }
        return eq;
    }

    /**
     * pokusi se nalezt predchozi job
     * nejprve hleda v aktualne ulozenych jobech
     * pokud nenajde -> hleda v databazi, nacteny job ulozi do jobsFromDB
     * @param id id zarizeni
     * @param curr aktualni radek
     * @return predchozi job
     */
    private HashMap<String, Object> getPreviousJob(int id, Key curr) {
        Key previous = new Key(Timestamp.valueOf(LocalDateTime.ofInstant(new Date(0L).toInstant(), ZoneId.systemDefault())), id);
        //nejdrive hleda v lokalnich datech
        for(Key key : Client.recalcedJobs.keySet()) {
            if(key.before(curr) && key.after(previous)) {
                previous = key;
            }
        }
        for(Key key : Client.jobsFromDB.keySet()) {
            if(key.before(curr) && key.after(previous)) {
                previous = key;
            }
        }
        HashMap<String, Object> row = new HashMap<>();
        //predchozi job nenalezen -> vytahnout z databaze
        if(previous.equals(new Key(Timestamp.valueOf(LocalDateTime.ofInstant(new Date(0L).toInstant(), ZoneId.systemDefault())), id))) {
            try (PreparedStatement preparedStatement = connection.prepareStatement("SELECT * FROM job WHERE id="+id+" limit -1", ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE)) {
                try (ResultSet rs = preparedStatement.executeQuery()) {
                    ResultSetMetaData md = rs.getMetaData();
                    int columns = md.getColumnCount();
                    while (rs.next()) {
                        for (int i = 1; i <= columns; ++i) {
                            row.put(md.getColumnName(i), rs.getObject(i));
                        }
                    }
                }
            } catch (SQLException throwable) {
                throwable.printStackTrace();
            }
        //nalezen
        } else {
            if(Client.recalcedJobs.containsKey(previous))
                return Client.recalcedJobs.get(previous);
            else
                return Client.jobsFromDB.get(previous);
        }
        //ulozit nacteny job pro mozna dalsi pouziti
        if(!row.isEmpty())
            Client.jobsFromDB.put(new Key((Timestamp) row.get("datetime"), (int) row.get("id")), row);
        return row;
    }

    /**
     * pokusi se nalezt data ze kterych se pocital predchozi job
     * nejprve hleda v aktualne ulozenych jobech
     * pokud nenajde -> hleda v databazi, nacteny job ulozi do jobDataFromDB
     * @param id id zarizeni
     * @param curr aktualni radek
     * @return data predchoziho jobu
     */
    private List<HashMap<String, Object>> getPreviousJobArray(int id, Key curr) {
        Key previous = new Key(Timestamp.valueOf(LocalDateTime.ofInstant(new Date(0L).toInstant(), ZoneId.systemDefault())), id);
        for(Key key : Client.jobs.keySet()) {
            if(key.before(curr) && key.after(previous)) {
                previous = key;
            }
        }
        for(Key key : Client.jobDataFromDB.keySet()) {
            if(key.before(curr) && key.after(previous)) {
                previous = key;
            }
        }
        List<HashMap<String, Object>> job = new ArrayList<>();
        if(previous.equals(new Key(Timestamp.valueOf(LocalDateTime.ofInstant(new Date(0L).toInstant(), ZoneId.systemDefault())), id))) {
           HashMap<String, Object> previousJob = getPreviousJob(id, curr);
           try (PreparedStatement preparedStatement = connection.prepareStatement("SELECT * FROM data WHERE id="+id+" AND data_time=to_timestamp('"+previousJob.get("datetime")+"', 'yyyy-MM-dd HH:mm:ss.S')", ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE)) {
                try (ResultSet rs = preparedStatement.executeQuery()) {
                    ResultSetMetaData md = rs.getMetaData();
                    int columns = md.getColumnCount();
                    while (rs.next()) {
                        HashMap<String, Object> row = new HashMap<>();
                        for (int i = 1; i <= columns; ++i) {
                            row.put(md.getColumnName(i), rs.getObject(i));
                        }
                        job.add(row);
                    }
                }
            } catch (SQLException throwable) {
                throwable.printStackTrace();
            }
        } else {
            if(Client.jobs.containsKey(previous))
                return Client.jobs.get(previous);
            else
                return Client.jobDataFromDB.get(previous);
        }
        if(!job.isEmpty())
            Client.jobDataFromDB.put(new Key((Timestamp) job.get(0).get("data_time"), (Integer) job.get(0).get("id")), job);
        return job;
    }

    /**
     * pokusi se nalezt joby v zadanem rozsahu
     * nejprve hleda v aktualne ulozenych jobech
     * pokud nenajde -> hleda v databazi, nacteny job ulozi do jobsFromDB_SUM
     * @return joby v zadanem rozsahu
     */
    private List<HashMap<String, Object>> getJobsInRange() {
        if(Client.jobsFromDB_SUM.containsKey((int) currRow)) {
            return Client.jobsFromDB_SUM.get(((int) currRow));
        }
        List<HashMap<String, Object>> jobs = new ArrayList<>();
        try (PreparedStatement preparedStatement = connection.prepareStatement("SELECT * FROM job WHERE datetime >= '"+Client.startStamp+"' AND data_time <= '"+Client.stopStamp+"'", ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE)) {
            try (ResultSet rs = preparedStatement.executeQuery()) {
                ResultSetMetaData md = rs.getMetaData();
                int columns = md.getColumnCount();
                while (rs.next()) {
                    HashMap<String, Object> row = new HashMap<>();
                    for (int i = 1; i <= columns; ++i) {
                        row.put(md.getColumnName(i), rs.getObject(i));
                    }
                    jobs.add(row);
                }
            }
        } catch (SQLException throwable) {
            throwable.printStackTrace();
        }
        if(!jobs.isEmpty())
            Client.jobsFromDB_SUM.put((int) currRow, jobs);
        return jobs;
    }

    /**
     * pokusi se nalezt predchozi den
     * nejprve hleda v aktualne ulozenych dnech
     * pokud nenajde -> hleda v databazi, nacteny den ulozi do daysFromDB
     * @return predchozi den
     */
    private HashMap<String, Object> getPreviousDay() {
        Key previous = new Key(Timestamp.valueOf(LocalDateTime.ofInstant(new Date(0L).toInstant(), ZoneId.systemDefault())), ((Key) currRow).getId());
        for(Key key : Client.recalcedDays.keySet()) {
            if(key.before((Key) currRow) && key.after(previous)) {
                previous = key;
            }
        }
        for(Key key : Client.daysFromDB.keySet()) {
            if(key.before((Key) currRow) && key.after(previous)) {
                previous = key;
            }
        }
        HashMap<String, Object> row = new HashMap<>();
        if(previous.equals(new Key(Timestamp.valueOf(LocalDateTime.ofInstant(new Date(0L).toInstant(), ZoneId.systemDefault())), ((Key) currRow).getId()))) {
            try (PreparedStatement preparedStatement = connection.prepareStatement("SELECT * FROM day WHERE id="+((Key) currRow).getId()+" limit -1", ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE)) {
                try (ResultSet rs = preparedStatement.executeQuery()) {
                    ResultSetMetaData md = rs.getMetaData();
                    int columns = md.getColumnCount();
                    while (rs.next()) {
                        for (int i = 1; i <= columns; ++i) {
                            row.put(md.getColumnName(i), rs.getObject(i));
                        }
                    }
                }
            } catch (SQLException throwable) {
                throwable.printStackTrace();
            }
        } else {
            if(Client.recalcedDays.containsKey(previous))
                return Client.recalcedDays.get(previous);
            else
                return Client.daysFromDB.get(previous);
        }

        if(!row.isEmpty())
            Client.daysFromDB.put(new Key((Timestamp) row.get("data_time"), (Integer) row.get("id")), row);
        return row;
    }

    /**
     * pokusi se nalezt joby ze kterych se pocital predchozi den
     * nejprve hleda v aktualne ulozenych jobech
     * pokud nenajde -> hleda v databazi, nactene joby ulozi do dayDataFromDB
     * @return joby predchoziho dne
     */
    private List<HashMap<String, Object>> getPreviousDayArray() {
        Key previous = new Key(Timestamp.valueOf(LocalDateTime.ofInstant(new Date(0L).toInstant(), ZoneId.systemDefault())), ((Key) currRow).getId());
        for(Key key : Client.daysFromJobs.keySet()) {
            if(key.before((Key) currRow) && key.after(previous)) {
                previous = key;
            }
        }
        for(Key key : Client.dayDataFromDB.keySet()) {
            if(key.before((Key) currRow) && key.after(previous)) {
                previous = key;
            }
        }
        List<HashMap<String, Object>> day = new ArrayList<>();
        if(previous.equals(new Key(Timestamp.valueOf(LocalDateTime.ofInstant(new Date(0L).toInstant(), ZoneId.systemDefault())), ((Key) currRow).getId()))) {
            HashMap<String, Object> previousDay = getPreviousDay();
            try (PreparedStatement preparedStatement = connection.prepareStatement("SELECT * FROM job WHERE id="+((Key) currRow).getId()+" AND data_time<to_timestamp('"+((Key) currRow).getData_time()+"', 'yyyy-MM-dd HH:mm:ss.S') AND datetime>=to_timestamp('"+previousDay.get("datetime")+"', 'yyyy-MM-dd HH:mm:ss.S')", ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE)) {
                try (ResultSet rs = preparedStatement.executeQuery()) {
                    ResultSetMetaData md = rs.getMetaData();
                    int columns = md.getColumnCount();
                    while (rs.next()) {
                        HashMap<String, Object> row = new HashMap<>();
                        for (int i = 1; i <= columns; ++i) {
                            row.put(md.getColumnName(i), rs.getObject(i));
                        }
                        day.add(row);
                    }
                }
            } catch (SQLException throwable) {
                throwable.printStackTrace();
            }
        } else {
            if(Client.daysFromJobs.containsKey(previous))
                return Client.daysFromJobs.get(previous);
            else
                return Client.dayDataFromDB.get(previous);
        }

        if(!day.isEmpty())
            Client.dayDataFromDB.put(new Key((Timestamp) day.get(0).get("data_time"), (Integer) day.get(0).get("id")), day);
        return day;
    }

    /**
     * pokusi se nalezt den pred zadanym rozsahem
     * nejprve hleda v aktualne ulozenych dnech
     * pokud nenajde -> hleda v databazi, nacteny den ulozi do daysFromDB_SUM
     * pro sumace
     * @return den pred rozsahem
     */
    private HashMap<String, Object> getDayBeforeRange() {
        if(Client.daysFromDB_SUM.containsKey((int) currRow)) {
            return Client.daysFromDB_SUM.get((int) currRow);
        }
        HashMap<String, Object> day = new HashMap<>();
        try (PreparedStatement preparedStatement = connection.prepareStatement("SELECT * FROM day WHERE datetime <= '"+Client.startStamp+"' limit -1", ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE)) {
            try (ResultSet rs = preparedStatement.executeQuery()) {
                ResultSetMetaData md = rs.getMetaData();
                int columns = md.getColumnCount();
                while (rs.next()) {
                    for (int i = 1; i <= columns; ++i) {
                        day.put(md.getColumnName(i), rs.getObject(i));
                    }
                }
            }
        } catch (SQLException throwable) {
            throwable.printStackTrace();
        }

        if(!day.isEmpty())
            Client.daysFromDB_SUM.put((int) currRow, day);
        return day;
    }

    /**
     * zpracuje nazev konstanty a preda getConstant()
     * @param eq nazev
     * @param id id zarizeni
     * @return hodnotu konstanty
     */
    private String getTableConstants(String eq, String id) {
        String variable = StringUtils.substringAfter(eq, "#");
        variable = StringUtils.substringBefore(variable, " ");
        HashMap<String, Object> constant = getConstant(id, variable);
        if(!constant.isEmpty()) {
            String value = constant.get("value").toString();
            eq = StringUtils.replaceOnceIgnoreCase(eq, "#"+variable, value);
        } else {
            eq = "";
        }
        return eq;
    }

    /**
     * nacte konstantu z databaze
     * nejprve hleda v aktualne ulozenych konstantach
     * pokud nenajde -> hleda v databazi, nactenou konstantu ulozi do constants
     * @param id id zarizeni
     * @param name jmeno konstanty
     * @return konstantu
     */
    public HashMap<String, Object> getConstant(String id, String name) {
        if(Client.constants.containsKey(name) && Client.constants.get(name).get("id").toString().equals(id)) {
            return Client.constants.get(name);
        }
        HashMap<String, Object> row = new HashMap<>();
        try (PreparedStatement preparedStatement = connection.prepareStatement("SELECT * FROM constants WHERE id="+id+" AND name='"+name+"'", ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE)) {
            try (ResultSet rs = preparedStatement.executeQuery()) {
                ResultSetMetaData md = rs.getMetaData();
                int columns = md.getColumnCount();
                while (rs.next()) {
                    for (int i = 1; i <= columns; ++i) {
                        row.put(md.getColumnName(i), rs.getObject(i));
                    }
                }
            }
        } catch (SQLException throwable) {
            throwable.printStackTrace();
        }

        if(!row.isEmpty())
            Client.constants.put(name, row);
        return row;
    }

    /**
     * kontroluje zda na zadane pozici je leve zavorka
     * pokud ano najde pozici odpovidajici prave zavorky
     * @param eq vzorecek
     * @param pos pozice
     * @return pokud existuje - pozice ukoncujici prave zavorky
     */
    private int checkForBrackets(String[] eq, int pos) {
        if(eq[pos].contains("(")) {
            int counter = 1;
            while (counter > 0) {
                if(pos+1 == eq.length)
                    return pos;
                String c = eq[++pos];
                if (c.contains("(")) {
                    counter++;
                }
                else if (c.contains(")")) {
                    counter--;
                }
            }
        }
        return pos;
    }

    /**
     * vytvori list desetinnych cisel z predaneho retezce
     * hodnoty oddeluje podle carky
     * @param string retezec cisel oddelene carkou
     * @return list desetinnych cisel
     */
    private List<Double> numberListFromArrayString(String string) {
        String[] strings;
        if(string.contains(", "))
            strings = string.replace("[", "").replace("]", "").split(", ");
        else
            strings = string.replace("[", "").replace("]", "").split(",");
        List<Double> result = new ArrayList<>();
        for (String s : strings) {
            if(isNumeric(s))
                result.add(Double.parseDouble(s));
        }
        return result;
    }

    /**
     * vytvori list retezcu z predaneho retezce
     * hodnoty oddeluje podle carky
     * @param string retezec retezcu oddelene carkou
     * @return list retezcu
     */
    private List<String> stringListFromArrayString(String string) {
        String[] strings;
        if(string.contains(", "))
            strings = string.replace("[", "").replace("]", "").split(", ");
        else
            strings = string.replace("[", "").replace("]", "").split(",");
        return new ArrayList<>(Arrays.asList(strings));
    }

    /**
     * prevede predany list desetinich cisel na retezec hodnot oddelenych carkou
     * @param list list ktery chceme prevest
     * @return retezec hodnot oddelenych carkou
     */
    private String doubleListToString(List<Double> list) {
        String s = list.toString();
        return s.replace("[", "").replace("]", "").replace(" ", "");
    }

    /**
     * prevede predany list celych cisel na retezec hodnot oddelenych carkou
     * @param list list ktery chceme prevest
     * @return retezec hodnot oddelenych carkou
     */
    private String intListToString(List<Integer> list) {
        String s = list.toString();
        return s.replace("[", "").replace("]", "").replace(" ", "");
    }

    /**
     * odstrani znkay = _ a ; z predaneho retezce
     * @param var retezce
     * @return upraveny retezec
     */
    private String removeLeadingAndEndingFromVariable(String var) {
        var = var.replace("=", "");
        var = var.replace("_", "");
        var = var.replace(StringUtils.substringAfter(var, ";"), "");
        return StringUtils.chop(var);
    }

    /**
     * testuje zda retezec je cislo
     * @param str retezec
     * @return true -> je cislo, false -> neni cislo
     */
    private boolean isNumeric(String str) {
        try {
            Double.parseDouble(str);
            return true;
        } catch(NumberFormatException e){
            return false;
        }
    }

    /**
     * testuje zda retezec je slozeny z velkych pismen
     * ignoruje cisla
     * @param s retezec
     * @return true -> vsechny pismena velka, false -> obsahuje mala pismena
     */
    private boolean isAllUpperCase(final String s) {
        if (StringUtils.isEmpty(s)) {
            return false;
        }
        final int sz = s.length();
        for (int i = 0; i < sz; i++) {
            if (!Character.isUpperCase(s.charAt(i))) {
                if(!Character.isDigit(s.charAt(i)))
                    return false;
            }
        }
        return true;
    }

    /**
     * vytvori list podle zadane osy histogramu
     * max min oddelene dvojteckou
     * pokud neobsahuje dvojtecku -> max min vypocte
     * hodnot oddelene carkou
     * @param input osa histogramu
     * @return list max/min danych skupin histogramu
     */
    private List<Double> parseHistogramAxis(String input) {
        List<Double> axis = new ArrayList<>();
        try {
            //negative infinity
            axis.add(-1e99);
            if(input.contains(":")) {
                String[] hilo_array = input.split(",");
                for(String hilo_part : hilo_array) {
                    String[] hilo_minmax = hilo_part.split(":");
                    for (String hiloMinmax : hilo_minmax) {
                        axis.add(Double.parseDouble(hiloMinmax));
                    }
                }
            } else {
                List<Double> hilo_temp = numberListFromArrayString(input);
                for(int i = 0; i<hilo_temp.size()-1; i++) {
                    axis.add((hilo_temp.get(i)+hilo_temp.get(i+1))/2);
                }
            }
            //positive infinity
            axis.add(1e99);
            return axis;
        } catch(NumberFormatException e){
            return axis;
        }
    }

    /**
     * testuje zda jmeno neodkazuje na sloupec
     * defaultne odkazuje na sloupec conjoined
     * @param variable retezec
     * @return jmeno sloupce
     */
    private String checkVariableName(String variable) {
        switch (variable.toLowerCase()) {
            case "datetime": return "datetime";
            case "data_time": return "data_time";
            case "id": return "id";
            case "state": return "state";
            case "user": return "user";
            case "gps": return "gps";
            case "recalc": return "recalc";
            default: return "conjoined";
        }
    }
}
