import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.sql.*;
import java.util.Objects;
import java.util.Properties;

/**
 * vytvori tabluky a naplni je testovacimi daty
 *
 * @author Petr Holicky
 */
public class SetupTestData {
    public static void main(String[] args) throws SQLException {
        new SetupTestData().setupDB();
    }

    private void setupDB() throws SQLException {
        Properties properties = new Properties();
        properties.setProperty("user", "admin");
        properties.setProperty("password", "quest");
        properties.setProperty("sslmode", "disable");
        final Connection connection = DriverManager.getConnection("jdbc:postgresql://localhost:8812/qdb", properties);

        PreparedStatement preparedStatement = connection.prepareStatement("CREATE TABLE data (datetime timestamp, data_time timestamp, id int, state int, user string, gps string, recalc int, conjoined string) timestamp(datetime) PARTITION BY hour;");
        preparedStatement.execute();
        preparedStatement = connection.prepareStatement("CREATE TABLE job (datetime timestamp, data_time timestamp, id int, state int, user string, gps string, recalc int, conjoined string) timestamp(datetime) PARTITION BY hour;");
        preparedStatement.execute();
        preparedStatement = connection.prepareStatement("CREATE TABLE day (datetime timestamp, data_time timestamp, id int, state int, user string, gps string, recalc int, conjoined string) timestamp(datetime) PARTITION BY hour;");
        preparedStatement.execute();
        preparedStatement = connection.prepareStatement("CREATE TABLE constants (id int, name string, value string);");
        preparedStatement.execute();
        preparedStatement = connection.prepareStatement("CREATE TABLE equations (type int, id int, name string, eq string);");
        preparedStatement.execute();

        System.out.println("TABLES CREATED");
        System.out.println("INSERTING DATA\nThis may take a while");

        PreparedStatement preparedStatementInsert = connection.prepareStatement("INSERT INTO data VALUES (to_timestamp(?, 'yyyy-MM-dd HH:mm:ss.S'), to_timestamp(?, 'yyyy-MM-dd HH:mm:ss.S'), ?, ?, ?, ?, ?, ?)");
        try (BufferedReader br = new BufferedReader(new InputStreamReader(Objects.requireNonNull(this.getClass().getResourceAsStream("/data.txt"))))) {
            String line;
            while ((line = br.readLine()) != null) {
                String[] parts = line.split("\t");
                preparedStatementInsert.setObject(1, parts[0]);
                preparedStatementInsert.setObject(2, parts[1]);
                preparedStatementInsert.setObject(3, parts[2]);
                preparedStatementInsert.setObject(4, parts[3]);
                preparedStatementInsert.setObject(5, parts[4]);
                preparedStatementInsert.setObject(6, parts[5]);
                preparedStatementInsert.setObject(7, parts[6]);
                preparedStatementInsert.setObject(8, parts[7]);
                preparedStatementInsert.addBatch();
            }
            preparedStatementInsert.executeBatch();
        } catch (IOException e) {
            e.printStackTrace();
        }

        preparedStatementInsert = connection.prepareStatement("INSERT INTO job VALUES (to_timestamp(?, 'yyyy-MM-dd HH:mm:ss'), to_timestamp(?, 'yyyy-MM-dd HH:mm:ss'), ?, ?, ?, ?, ?, ?)");
        try (BufferedReader br = new BufferedReader(new InputStreamReader(Objects.requireNonNull(this.getClass().getResourceAsStream("/job.txt"))))) {
            String line;
            while ((line = br.readLine()) != null) {
                String[] parts = line.split("\t");
                preparedStatementInsert.setObject(1, parts[0]);
                preparedStatementInsert.setObject(2, parts[1]);
                preparedStatementInsert.setObject(3, parts[2]);
                preparedStatementInsert.setObject(4, parts[3]);
                preparedStatementInsert.setObject(5, parts[4]);
                preparedStatementInsert.setObject(6, parts[5]);
                preparedStatementInsert.setObject(7, parts[6]);
                preparedStatementInsert.setObject(8, parts[7]);
                preparedStatementInsert.addBatch();
            }
            preparedStatementInsert.executeBatch();
        } catch (IOException e) {
            e.printStackTrace();
        }

        preparedStatementInsert = connection.prepareStatement("INSERT INTO day VALUES (to_timestamp(?, 'yyyy-MM-dd HH:mm:ss'), to_timestamp(?, 'yyyy-MM-dd HH:mm:ss'), ?, ?, ?, ?, ?, ?)");
        try (BufferedReader br = new BufferedReader(new InputStreamReader(Objects.requireNonNull(this.getClass().getResourceAsStream("/day.txt"))))) {
            String line;
            while ((line = br.readLine()) != null) {
                String[] parts = line.split("\t");
                preparedStatementInsert.setObject(1, parts[0]);
                preparedStatementInsert.setObject(2, parts[1]);
                preparedStatementInsert.setObject(3, parts[2]);
                preparedStatementInsert.setObject(4, parts[3]);
                preparedStatementInsert.setObject(5, parts[4]);
                preparedStatementInsert.setObject(6, parts[5]);
                preparedStatementInsert.setObject(7, parts[6]);
                preparedStatementInsert.setObject(8, parts[7]);
                preparedStatementInsert.addBatch();
            }
            preparedStatementInsert.executeBatch();
        } catch (IOException e) {
            e.printStackTrace();
        }

        preparedStatementInsert = connection.prepareStatement("INSERT INTO constants VALUES (?, ?, ?)");
        preparedStatementInsert.setObject(1, 11);
        preparedStatementInsert.setObject(2, "TMAX");
        preparedStatementInsert.setObject(3, 2);
        preparedStatementInsert.addBatch();
        preparedStatementInsert.setObject(1, 11);
        preparedStatementInsert.setObject(2, "WB");
        preparedStatementInsert.setObject(3, "080000");
        preparedStatementInsert.addBatch();
        preparedStatementInsert.executeBatch();

        preparedStatementInsert = connection.prepareStatement("INSERT INTO equations VALUES (?, ?, ?, ?)");
        try (BufferedReader br = new BufferedReader(new InputStreamReader(Objects.requireNonNull(this.getClass().getResourceAsStream("/equations.txt"))))) {
            String line;
            while ((line = br.readLine()) != null) {
                String[] parts = line.split("\t");
                if(parts[0].equals("1")) {
                    preparedStatementInsert.setObject(1, parts[0]);
                    preparedStatementInsert.setObject(2, parts[1]);
                    preparedStatementInsert.setObject(3, parts[2]);
                    preparedStatementInsert.setObject(4, "");
                    preparedStatementInsert.addBatch();
                } else {
                    preparedStatementInsert.setObject(1, parts[0]);
                    preparedStatementInsert.setObject(2, parts[1]);
                    preparedStatementInsert.setObject(3, parts[2]);
                    preparedStatementInsert.setObject(4, parts[3]);
                    preparedStatementInsert.addBatch();
                }
            }
            preparedStatementInsert.executeBatch();
        } catch (IOException e) {
            e.printStackTrace();
        }
        System.out.println("ALL DATA SUCCESSFULLY INSERTED");
    }
}
